// com8001_prbs11.cpp : Generates a (large) binary file for testing the COM-8001 
// arbitrary waveform generator. The file consists of PRBS-11 sequence (pseudo-random
// binary sequence, 2047-bit period), formatted for a 1-BIT WIDE OUTPUT or 2-BIT WIDE OUTPUT
// samples, for interface with a (COM-1002) QPSK modulator, 
// or (COM-1005) bit error rate measurement module for example.

#include <math.h>
#include <stdio.h>


int main(int argc, char* argv[])
{
	long il,ilmax;
	FILE *stream;
	unsigned char byteout;
	short numwritten;
	unsigned char PRBS11[2047];
	unsigned char LFSR[11];
	unsigned char LFSR_feedback;
	unsigned short i,j, output_width;
	unsigned char D[8];
	
	ilmax = (2047*32)*16;	// size of file in bytes 
	// must be an integer multiple of 2047 BITS to avoid bit errors 
	// when rewinding playback read pointer.
	// must be an integer of 32 bytes (COM-8001 constraint)
	// 1,048,064 bytes

	output_width = 1;	// select 1 for 1-bit wide, 2 for 2-bit wide output samples
						// 8 for 2-bit wide with enable and start of period test point.
	
	printf("Generating PRBS11 test file for the COM-8001\n");

	// initialize PRBS11 sequence with all 0's
	for(i = 0; i < sizeof(LFSR); i++){
		LFSR[i] = 0;
	}
	// PRBS11 sequence generation, bit by bit. See COM-1005 specifications for taps definition
	for(i = 0; i< sizeof(PRBS11); i++){
		PRBS11[i] = LFSR[10];
		LFSR_feedback = (~(LFSR[10] ^ LFSR[8])) & 0x01;
		for(j = 10; j >0; j--)
			LFSR[j] = LFSR[j-1];
		LFSR[0] = LFSR_feedback;
		
	}


	if((stream = fopen("binary_file.bin","w+b")) == NULL)
		printf("the file 'binary_file.dat' was not opened\n");

	// Waveform generation 


	j = 0;	// PRBS11 read pointer
	for(il = 0; il<ilmax; il++){

		if(output_width == 1){
			// 1-bit wide output samples
			// Byte packing format is (MSb)[I0 I1 I2 I3 I4 I5 I6 I7](LSb)
			// where I is transmitted on DATA_OUT pin J4/A2	
			byteout = 0x00;	// initialize byte
			for(i = 0; i < 8; i++){
				byteout = (byteout << 1);	// first bit is MSb
				if(PRBS11[j]) 
					byteout |= 0x01;		// first bit is MSb
				// modulo 2047 PRBS11 read pointer
				if(j < 2046)
					j++;
				else
					j = 0;
			}
		} else if(output_width == 2){
			// 2-bit wide output samples
			// Byte packing format is (MSb)[Q0 I0  Q1 I1  Q2 I2  Q3 I3](LSb)
			// where I is transmitted on DATA_OUT(0) pin J4/A2	
			// and Q is transmitted on DATA_OUT(1) pin J4/B2
			
			// get next eight bits D7...D0
			for(i = 0; i < 8; i++){
				D[i] = PRBS11[j];
				// modulo 2047 PRBS11 read pointer
				if(j < 2046)
					j ++;
				else
					j = 0;
			}


			byteout = 0x00;	// initialize byte
			if(D[1]) 
				byteout |= 0x80;		// MSB	
			if(D[0]) 
				byteout |= 0x40;		
			if(D[3]) 
				byteout |= 0x20;		
			if(D[2]) 
				byteout |= 0x10;		
			if(D[5]) 
				byteout |= 0x08;		
			if(D[4]) 
				byteout |= 0x04;		
			if(D[7]) 
				byteout |= 0x02;		
			if(D[6]) 
				byteout |= 0x01;		// LSB

		} else if(output_width == 8){
			// 8-bit wide output samples containing two data bits (I,Q), plus ENABLE, plus Start Of Frame oscilloscope trigger (test)
			// Byte packing format is (MSb)[ENABLE x x x SOFtestpoint x Q0 I0](LSb)
			// where I is transmitted on DATA_OUT(0) pin J4/A2	
			// and Q is transmitted on DATA_OUT(1) pin J4/B2
			// and SOFtestpoint is transmitted on DATA_OUT(3) pin J4/B3
			// and ENABLE is transmitted on DATA_OUT(7) pin J4/A6
			
			// get next two bits D1,D0
			for(i = 0; i < 2; i++){
				D[i] = PRBS11[j];
				// modulo 2047 PRBS11 read pointer
				if(j < 2046)
					j ++;
				else
					j = 0;
			}


			byteout = 0x80;	// initialize byte. Make sure to drive ENABLE to '1'.
			if(D[1]) 
				byteout |= 0x02;		// LSB
			if(D[0]) 
				byteout |= 0x01;		// LSB
			if(j == 0)
				byteout |= 0x08;		// SOF (test point), every 2*2047 bits

		}

		numwritten = fwrite(&byteout,1,1,stream);

	}

	fclose(stream);
	// Format conversion


	return 0;
}

